# utility to get receiver positions from offsets 181 and 185 in the segy file (first break picks) and output to a text file.
getrecxy-fb: ${S}/getrecxy-fb.pl
	${S}/getrecxy.pl ${MVDATA} ${D}

#================================
# run a perl script that does the segyread from the original microspread SEGY files, then Su2sep for each node's file, and then Cats them together.
${D}/microspread.H: ${S}/importdata.pl
	${S}/importdata.pl ${MVDATA} ${D}
#================================


# shift positions to start from 0 and convert to Km. Convert everything to floats.
${D}/microspread2.H:
	Headermath < ${D}/microspread.H par=${P}/ms.P > $@ hff=$@@@

# Get rid of keys I don't think we need.
${D}/microspread-delkeys.H: ${D}/microspread2.H
	Headermath < $< key1=Dummy type1=scalar_float eqn1=tracl+0. delete_keys=31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54 > $@ hff=$@@@

# need to do this to fool seplib so I can use window_key.
# This is because the file we get from concatenating the different node's .H files generates a file that has n3=26. Window_key doesn't like this.
# We set n2 = n2*n3
${D}/microspread3.H: ${D}/microspread-delkeys.H
	cp $< $@
	cp $<@@ $@@@
	echo n3=1 >> $@
	echo n2=226824 >> $@
	echo n3=1 >> $@@@
	echo n2=226824 >> $@@@
	cat $@ | grep -v hff > tmp
	echo hff="$@@@" >> tmp
	cp tmp $@

# remap first-break node positions taken from data files to correct for errors in the relative node positions.
# This was done by observing the relative node positions in the text file we got from Paul Milligan at Seabed Geosolutions (p11_rcvpos_ALL.txt)
#${D}/microspread4.H: <---- see mk/remap.mk

# generate x and y offset keys with the corrected FB receiver positions
${D}/microspread5.H: ${D}/microspread4.H
	Headermath < $< key1=Offsetx type1=scalar_float eqn1="Sx-Gxfb" key2=Offsety type2=scalar_float eqn2="Sy-Gyfb" > $@ hff=$@@@

# change the polarity of the Vx and Vy components of the southern node line so it will match with the polarity of the northern node line.
${D}/microspread6.H: ${D}/microspread5.H ${B}/fix_pol.x
	${B}/fix_pol.x < $< gyflip=0.006 > $@ hff=$@@@


# receivers are at 27.338 km till 27.362 km.
# We make a shot gather with a source that is to the West of the receivers.
# The X component is positive toward the East, therefore ,fix_pol.x should have made it so that the first break is positive on all of the traces on this gather.
${R}/test-vx-pol.v: ${D}/microspread5.H ${D}/microspread6.H
	Window_key < ${D}/microspread5.H key1=Sailline mink1=2001 maxk1=2001 key2=Tracf mink2=0.9 maxk2=1.1 key3=Sx mink3=25.02 maxk3=25.03 synch=1 1> ${D}/1300vxb.H hff=${D}/1300vxb.H@@ 2>${dn}
	Window_key < ${D}/microspread6.H key1=Sailline mink1=2001 maxk1=2001 key2=Tracf mink2=0.9 maxk2=1.1 key3=Sx mink3=25.02 maxk3=25.03 synch=1 1> ${D}/1300vxa.H hff=${D}/1300vxa.H@@ 2>${dn}
	echo "hff=-1" >> ${D}/1300vxb.H
	echo "hff=-1" >> ${D}/1300vxa.H
	Merge axis=3 space=n ${D}/1300vxb.H ${D}/1300vxa.H | Window3d min1=1.8 max1=2.1 | ${Wiggly} out=$@ > ${dn}
	Rm ${D}/1300vxb.H ${D}/1300vxa.H
	


#===============================
# shot-receiver positions
#===============================

# shot positions for the two separate sail lines
${D}/sxsy1.H: ${D}/microspread5.H
	Window_key < $< key1=Tracf mink1=0.5 maxk1=1.5 key2=Sailline mink2=2000.5 maxk2=2001.5 1> ${D}/sail1.H hff=${D}/sail1.H@@ 2>${dn}
	Window3d < ${D}/sail1.H@@ n1=1 f1=42 > ${D}/Sx1.H
	Window3d < ${D}/sail1.H@@ n1=1 f1=43 > ${D}/Sy1.H
	Cmplx ${D}/Sx1.H ${D}/Sy1.H > $@

${D}/sxsy2.H: ${D}/microspread5.H
	Window_key < $< key1=Tracf mink1=0.5 maxk1=1.5 key2=Sailline mink2=2001.5 maxk2=2002.5 1> ${D}/sail2.H hff=${D}/sail2.H@@ 2>${dn}
	Window3d < ${D}/sail2.H@@ n1=1 f1=42 > ${D}/Sx2.H
	Window3d < ${D}/sail2.H@@ n1=1 f1=43 > ${D}/Sy2.H
	Cmplx ${D}/Sx2.H ${D}/Sy2.H > $@

# receiver positions from as-laid coordinates, which look wrong.
${D}/gxgy-aslaid.H: ${D}/microspread-delkeys.H
	Window3d < $<@@ n1=1 f1=45 n2=1 f2=0 > ${D}/Gx.H
	Window3d < $<@@ n1=1 f1=46 n2=1 f2=0 > ${D}/Gy.H
	Cmplx ${D}/Gx.H ${D}/Gy.H > $@

# receiver positions from first-break picking coordinates, which look really regular, almost doctored.
# however, after observing the ROV videos, these positions seem to be pretty reasonable, so I shall use them rather than the as-laid coordinates.
# please note that the node positions are wrong in this file. The correct ones are in dat/microspread4.H. However, their geometry looks the same in both files.
${D}/gxgy-fb.H: ${D}/microspread-delkeys.H
	Window3d < $<@@ n1=1 f1=47 n2=1 f2=0 > ${D}/Gxfb.H
	Window3d < $<@@ n1=1 f1=48 n2=1 f2=0 > ${D}/Gyfb.H
	Cmplx ${D}/Gxfb.H ${D}/Gyfb.H > $@


#===============================
# shot-receiver figures
#===============================
${R}/gxgy-aslaid.v: ${D}/gxgy-aslaid.H
	Graph < $< symbol='*' symbolsz=10 plotcol=${yel} title="As-laid receiver positions"  label1="X(km)" label2="Y(km)" out=$@ > ${dn}

${R}/gxgy-fb.v: ${D}/gxgy-fb.H
	Graph < $< symbol='*' symbolsz=10 plotcol=${red} title="First-break receiver positions"  label1="X(km)" label2="Y(km)" out=$@ > ${dn}

${R}/gxgy-all.v: ${D}/gxgy-aslaid.H ${D}/gxgy-fb.H
	Merge axis=2 space=n $^ | Graph symbol="+o" symbolsz=10 grid=n min2=0. max2=0.012 d1num=0.006 d2num=0.004 o1num=27.338 o2num=0 legendpos=8.5,1.5 title="Microspread receiver positions" label1="X (km)" label2="Y (km)" legend=y curvelabel="As-laid positions:First break positions" out=$@ > ${dn}

${R}/src-rec.v: ${D}/sxsy1.H ${D}/sxsy2.H ${D}/gxgy-fb.H
	Graph < ${D}/sxsy1.H symbol="*"   plotcol=${yel} symbolsz=10 grid=n min1=26.4 max1=28.4 min2=0 max2=0.012 titlesz=0 labelsz=0 out=${R}/tmp1.v > ${dn}
	Graph < ${D}/sxsy2.H symbol="*"   plotcol=${red} symbolsz=10 grid=n min1=26.4 max1=28.4 min2=0 max2=0.012 titlesz=0 labelsz=0 out=${R}/tmp2.v > ${dn}
	Graph < ${D}/gxgy-fb.H symbol="o" plotcol=${cya} symbolsz=10 grid=n min1=26.4 max1=28.4 min2=0 max2=0.012 title="near-shots (*) and receivers (o)" label1="X (km)" label2="Y (km)" out=${R}/tmp3.v > ${dn}
	vp_Overlay ${R}/tmp3.v ${R}/tmp2.v ${R}/tmp1.v > $@
	-rm ${R}/tmp[123].v


importgathers: ${D}/microspread.H ${D}/microspread2.H ${D}/microspread-delkeys.H ${D}/microspread3.H ${D}/microspread4.H ${D}/microspread5.H ${D}/sxsy1.H ${D}/sxsy2.H ${D}/gxgy-aslaid.H ${D}/gxgy-fb.H
	make ${R}/gxgy-all.v ${R}/src-rec.v
	make ${R}/gxgy-all.pdf ${R}/src-rec.pdf

