program desig
	
	use sep
	use base_fftwfl_mod
	use base_fftwv3_mod
	implicit none
	
	integer, external	:: sreed, srite
	integer				:: n1,n2,nt,nf,n1_sig
	real				:: d1,o1,dt,ot,df,of,d1_sig
	integer				:: b,e
	integer				:: ii
	
	real,allocatable	:: gdata(:),gsig_hyd(:),gsig_geo(:),taper(:)
	complex,allocatable	:: tdata(:),fdata(:),fdata_amp(:)
	complex,allocatable	:: tsig_hyd(:),fsig_hyd(:),fsig_hyd_amp(:)
	complex,allocatable	:: tsig_geo(:),fsig_geo(:),fsig_geo_amp(:)
	real,allocatable :: header(:)
	
	integer*8			:: fft_for,fft_inv,fft_sig_hyd,fft_sig_geo
	logical				:: true_t,verbose
	
	real, parameter		:: pi=3.1415926535
	
	real				:: f,w,heps,zeps,xeps,yeps,eps_sig_hyd,eps_sig_geo,eps_data,ftnorm

	real				:: delt
	integer				:: delti,rc

	character*64		:: compkey ! name of key that has component id
	integer				:: compidx ! index of key that has component id
	integer				:: nhead	! number of header keys
	integer				:: ihead	! index of current trace

	integer				:: hyd_compid, vz_compid, vx_compid, vy_compid


	call sep_init()
	call init_3d ()
	call from_param('verbose',verbose,.false.)
	call from_param('verb'	 ,verbose,verbose)
	if (verbose) write(0,*) '-------------------------------------------'
	if (verbose) write(0,*) '				 GEOPHONE DESIGNATURE							'
	if (verbose) write(0,*) '-------------------------------------------'
	
	call from_history('n1',n1,1 )
	call from_history('n2',n2,1 )
	call from_history('d1',d1,1.)
	call from_history('o1',o1,0.)

	call from_param ('heps',heps,.001)
	call from_param ('zeps',zeps,.001)
	call from_param ('xeps',xeps,.001)
	call from_param ('yeps',yeps,.001)

	call from_param ('compkey', compkey, 'Tracf')
	call from_param ('hyd', hyd_compid, 4)
	call from_param ('vz',  vz_compid,  3)
	call from_param ('vx',  vx_compid,  1)
	call from_param ('vy',  vy_compid,  2)

	call from_aux('hsig', 'n1', n1_sig)
	call from_aux('hsig', 'd1', d1_sig)
	if (n1_sig .ne. n1)           call seperr ("n1 of data does not equal n1 of hydrophone instrument signature")
	if (abs(d1-d1_sig) .gt. 1e-5) call seperr ("d1 of data does not equal d1 of hydrophone instrument signature")

	call from_aux('gsig', 'n1', n1_sig)
	call from_aux('gsig', 'd1', d1_sig)
	if (n1_sig .ne. n1)           call seperr ("n1 of data does not equal n1 of geophone instrument signature")
	if (abs(d1-d1_sig) .gt. 1e-5) call seperr ("d1 of data does not equal d1 of geophone instrument signature")

	call sep_get_key_index   ("in", compkey, compidx)
	call sep_get_number_keys ("in", nhead)
	
	call from_param('true_t',true_t,.true.)
	if (verbose) write(0,*) 'true_t',true_t

	! Determine how to treat time axis. true_t means to actually consider the value of o1 from the input.
	if (true_t) then
		ot = minval( (/o1,-abs(o1+n1*d1)/) )
		dt = d1
		nt = nint(-ot/d1*2)	! this effectively does zero-padding, with the most extreme time available in the data reaching one end.
		b	= nint((o1-ot)/d1)+1	! index where the data begins in the input array
		e	= b+n1-1		! index where the data ends in the input array
	else
		ot=	0.
		dt= d1
		nt= n1
		b =	1
		e = b+n1-1
	end if

	! Fourier units
	df = 1./(nt*dt)
	nf = nt
	of = -nf/2.*df

	ftnorm = 1./sqrt(1.*nf)

	if (verbose) write(0,*) 'n1,d1,o1',n1,d1,o1
	if (verbose) write(0,*) 'nt,dt,ot',nt,dt,ot
	if (verbose) write(0,*) 'nf,df,of',nf,df,of
	if (verbose) write(0,*) 'Defining FT axis as (-t,t): ',ot,ot+(nt-1)*d1
	if (verbose) write(0,*) 'read data into indices: ',b,e,'out of',nt
	
	call from_param('delt',delt,.120) ! taper length

	call sep_close()
	
	allocate (gdata(n1), tdata(nt), fdata(nf))
	allocate (gsig_hyd(n1),  tsig_hyd(nt),  fsig_hyd(nf))
	allocate (gsig_geo(n1),  tsig_geo(nt),  fsig_geo(nf))
	allocate (fsig_hyd_amp(nf), fsig_geo_amp(nf), fdata_amp(nf))
	allocate (taper(n1))
	allocate (header (nhead))

	call sfftw_plan_dft_1d(fft_for, nt,	tdata, fdata, FFTW_FORWARD,  FFTW_ESTIMATE);
	call sfftw_plan_dft_1d(fft_inv, nt,	fdata, tdata, FFTW_BACKWARD, FFTW_ESTIMATE);
	call sfftw_plan_dft_1d(fft_sig_hyd, nt,	tsig_hyd,  fsig_hyd,  FFTW_FORWARD,  FFTW_ESTIMATE);
	call sfftw_plan_dft_1d(fft_sig_geo, nt,	tsig_geo,  fsig_geo,  FFTW_FORWARD,  FFTW_ESTIMATE);

	! Construct Taper
	delti=nint(delt/d1)
	if (verbose) write(0,*) 'delti',delti,delt,d1
	taper=1;
	if (delti.gt.0) then
		do ii=1,delti
			taper(ii)= 1-.5*(1.+cos(pi*(ii-1)/real(delti)))
		end do
		do ii=1,delti
			taper(n1-ii+1)= 1-.5*(1.+cos(pi*(ii-1)/real(delti)))
		end do
	end if

	! Taper as output
	if (exist_file("taper")) then
		if (verbose) write(0,*) 'write taper to file'
		call to_history("n1", n1 ,"taper")
		call to_history("d1", d1 ,"taper")
		call to_history("o1", o1 ,"taper")
		if (srite("taper",taper,4*n1) .eq. 0) call seperr ("error writing taper");
	end if	

	if (exist_file("fsig_hyd")) then
		if (verbose) write(0,*) 'write signature spectrum to file'
		call to_history("n1", nf ,"fsig_hyd")
		call to_history("d1", df ,"fsig_hyd")
		call to_history("o1", of ,"fsig_hyd")
	end if	

	if (exist_file("fsig_geo")) then
		if (verbose) write(0,*) 'write signature spectrum to file'
		call to_history("n1", nf ,"fsig_geo")
		call to_history("d1", df ,"fsig_geo")
		call to_history("o1", of ,"fsig_geo")
	end if	

	!======================================
	! Prepare hydrophone instrument signature
	gsig_hyd = 0.; tsig_hyd=0.; fsig_hyd=0.;
	if (sreed ("hsig" , gsig_hyd, n1*4) .eq. 0) call seperr ("error reading input data");
	tsig_hyd (b:e) = gsig_hyd
	
	! shift signature and FT.
	tsig_hyd = cshift (tsig_hyd, shift=nt/2-1)
	call sfftw_execute (fft_sig_hyd)

	! normalize signature amplitude
	eps_sig_hyd  = heps*(sum(cabs(fsig_hyd)))/nf
	fsig_hyd_amp = sqrt (fsig_hyd * conjg(fsig_hyd))
	fsig_hyd     = fsig_hyd / (fsig_hyd_amp + eps_sig_hyd)
	
	if (exist_file("fsig_hyd")) then
		if (srite ("fsig_hyd", cabs(fsig_hyd)*ftnorm, 4*size(fsig_hyd)) .eq. 0) call seperr ("error writing designature function spectrum");
	endif

	!======================================
	! Prepare geophone instrument signature
	gsig_geo = 0.; tsig_geo=0.; fsig_geo=0.;
	if (sreed ("gsig" , gsig_geo, n1*4) .eq. 0) call seperr ("error reading input data");
	tsig_geo (b:e) = gsig_geo
	
	! shift signature and FT.
	tsig_geo = cshift (tsig_geo, shift=nt/2-1)
	call sfftw_execute (fft_sig_geo)

	! normalize signature amplitude
	eps_sig_geo  = zeps*(sum(cabs(fsig_geo)))/nf
	fsig_geo_amp = sqrt (fsig_geo * conjg(fsig_geo))
	fsig_geo     = fsig_geo / (fsig_geo_amp + eps_sig_geo)
	
	if (exist_file("fsig_geo")) then
		if (srite ("fsig_geo", cabs(fsig_geo)*ftnorm, 4*size(fsig_geo)) .eq. 0) call seperr ("error writing designature function spectrum");
	endif

	if (verbose) then
		write (0,*), 'zeps', zeps
		write (0,*), 'eps_sig_geo', eps_sig_geo
		write (0,*), 'heps', heps
		write (0,*), 'eps_sig_hyd', eps_sig_hyd
	endif

	!======================================
	! go over traces and apply hydrophone/geophone designature
	ihead = 1
	gdata = 0.
	rc = sreed ("in", gdata, n1*4)
	do while (rc .eq. n1*4)

		tdata=cmplx(0., 0.); fdata=cmplx (0., 0.); fdata_amp=cmplx (0., 0.)

		tdata(b:e)= cmplx(gdata*taper, 0.)

		! shift and FT transform data. The negative times need to be placed at the second half of the array.
		tdata = cshift (tdata, shift=nt/2-1)
		call sfftw_execute( fft_for )

		! read header values of this trace
		call sep_get_val_headers("in", ihead, 1, header)

		! apply hydrophone designature
		if (nint(header (compidx)) .eq. hyd_compid) then
			eps_data  = heps*(sum(cabs(fdata)))/nf
			fdata_amp = sqrt (fdata * conjg(fdata))
			fdata     = fdata / (fdata_amp + eps_data)

			fdata = conjg (fsig_hyd) * fdata * ftnorm * (fdata_amp / (fsig_hyd_amp + eps_data))

		! apply vertical geophone designature
		elseif (nint(header (compidx)) .eq. vz_compid) then
			eps_data  = zeps*(sum(cabs(fdata)))/nf
			fdata_amp = sqrt (fdata * conjg(fdata))
			fdata     = fdata / (fdata_amp + eps_data)

			fdata = conjg (fsig_geo) * fdata * ftnorm * (fdata_amp / (fsig_geo_amp + eps_data))

		! apply vertical geophone designature
		elseif (nint(header (compidx)) .eq. vx_compid) then
			eps_data  = xeps*(sum(cabs(fdata)))/nf
			fdata_amp = sqrt (fdata * conjg(fdata))
			fdata     = fdata / (fdata_amp + eps_data)

			fdata = conjg (fsig_geo) * fdata * ftnorm * (fdata_amp / (fsig_geo_amp + eps_data))

		! apply vertical geophone designature
		elseif (nint(header (compidx)) .eq. vy_compid) then
			eps_data  = yeps*(sum(cabs(fdata)))/nf
			fdata_amp = sqrt (fdata * conjg(fdata))
			fdata     = fdata / (fdata_amp + eps_data)

			fdata = conjg (fsig_geo) * fdata * ftnorm * (fdata_amp / (fsig_geo_amp + eps_data))
		endif

		! FT inverse transform data and shift back.
		call sfftw_execute( fft_inv )
		tdata = cshift (tdata, shift=-nt/2+1)
		gdata = tdata(b:e)/sqrt(REAL(nt))

		if (srite ("out" , gdata, n1*4) .eq. 0) call seperr ("error writing output data");

		if (mod (ihead, 10000) .eq. 0 .and. verbose) then
			write (0,*) 'Processed trace:', ihead
		endif

		gdata = 0.
		rc = sreed ("in", gdata, n1*4)
		ihead = ihead + 1
	end do


	if (verbose) write(0,*) '-------------------------------------------'
	call sfftw_destroy_plan(fft_for,fft_inv,fft_sig_hyd,fft_sig_geo)
	deallocate (gdata,tdata,fdata,fdata_amp)
	deallocate (gsig_hyd,tsig_hyd,fsig_hyd,fsig_hyd_amp)
	deallocate (gsig_geo,tsig_geo,fsig_geo,fsig_geo_amp)
	deallocate (taper)
	call exit(0)

end program

