!!$
!!$=head1 NAME
!!$
!!$nmo_mod - Perform nmo
!!$
!!$=head1 SYNOPSIS
!!$
!!$Initializer-C<call nmo_init(slowness,offset,o1,d1)>
!!$
!!$operator-C<call nmo_nearest(adj,tt,zz)>
!!$
!!$operator-C<call nmo_linear(adj,tt,zz)>
!!$
!!$=head1 INITIALIZER
!!$
!!$=head2 INPUT PARAMETERS
!!$
!!$=over 4
!!$
!!$=item slowness - C<real(:)>
!!$
!!$           Slowness(t)
!!$
!!$=item offset   - C<real(:)>
!!$
!!$           Offset
!!$
!!$=item o1       - real
!!$
!!$           First sample in time
!!$
!!$=item d1       - real
!!$
!!$           Sampling in time
!!$
!!$=item inverse  - logical
!!$
!!$           Wheter or not to do inverse
!!$
!!$=back
!!$
!!$=head1  OPERATORS
!!$
!!$=over 4
!!$
!!$=item adj   - logical
!!$
!!$        Whether or not do adjoint
!!$
!!$=item tt    - C<real(:,:)>
!!$
!!$        Input
!!$
!!$=item zz    - C<real(:,:)>
!!$
!!$        Output
!!$
!!$=back
!!$
!!$
!!$=head1 DESCRIPTION
!!$Nmo by nearest or linear interpolation neighbor for a gather
!!$
!!$=head1 COMMENTS
!!$
!!$NMO, its transpose, inverse, and inverse transpose by nearest neighbor.
!!$
!!$adj=.false. inv=.false.: zz(iz) = [NMO]                   tt(it)
!!$
!!$adj=.true. inv=.false.: tt(it) = [NMO adj]         zz(iz)
!!$
!!$adj=.false. inv=.true.:        tt(it) = [NMO inverse]           zz(iz)
!!$
!!$adj=.true. inv=.true.:        zz(iz) = [NMO inverse transpose] tt(it)
!!$
!!$
!!$=head1 SEE ALSO
!!$
!!$L<Nmo3d>
!!$
!!$=head1 LIBRARY
!!$
!!$B<sepvelanf90>
!!$
!!$=cut
module mynmo_mod 
	implicit none
	real,save,private :: slow
	real,save, pointer,private :: x(:) !offset
	real,save,private :: t0,dt
	integer,save,private :: nt,n2
	logical, save,private :: inv
	contains
	subroutine nmo_init(slowness,offset,n1,o1,d1,inverse,first)
		real, intent (in)	:: offset(:)
		real, intent (in)			:: slowness
		integer, intent (in)		:: n1
		real, intent (in)			:: o1, d1
		logical, intent (in)		:: inverse
		logical, intent (inout)		:: first

		inv=inverse
		t0=o1
		dt=d1
		nt=n1
		slow=slowness
		if (.not. first) then
			deallocate(x)
		end if
		allocate(x(size(offset)))
		x=offset
		first=.false.
		n2=size(x)
	end subroutine 
!>
!
	subroutine nmo_nearest (adj, tt, zz)
		logical :: adj
		integer   it, iz, off, last,i2
		real   tt(:,:), zz(:,:)
		real t, z, xs
		integer :: itab(nt)
		real :: count(nt)
		if (inv) then
			count=0.
		end if
		do i2=1,n2 
			z = t0 + nt * dt
			t = z
			off = nt + 1
			do iz = nt, 1, -1   
				xs = x(i2) * slow
				t = sqrt (z * z + xs * xs)
				it = 0.5 + (t - t0) / dt
				if ( it <= nt  ) then
					count(it) = count(it)+ 1.
					itab(iz) = it
				else
					off = iz
				end if 
				z = z - dt
			end do 
			last = off - 1
			if (adj .and. inv) then
				where(count.ne.0.)
					tt(:,i2) = tt(:,i2)/count
				end where
			end if
			if ( (adj .and. inv) .or. (.not. adj .and. .not. inv) ) then
				do iz = 1, last  
					it = itab(iz)
					zz(iz,i2) = zz(iz,i2) + tt(it,i2)
				end do
			else
				do iz = 1, last 
					it = itab(iz)
					tt(it,i2) = tt(it,i2)+ zz(iz,i2)
				end do
			end if 
			if (.not. adj .and. inv) then
				where(count.ne.0.)
					tt(:,i2) = tt(:,i2)/count
				end where
			end if
		end do
	end subroutine 
end module 
